/******************************************************************************
 * Generic TCP Client Example for Microchip TCP/IP Stack                      *
 *============================================================================*
 *                                                                            *
 * This module implements an example HTTP client and can be used as a         *
 * generic reference for a TCP Client implementation.                         *
 *                                                                            *
 * The HTTP client establishes a TCP connection to www.google.com submits a   *
 * search URL and shows the results via the serial interface.                 *
 *                                                                            *
 * SOFTWARE LICENSE AGREEMENT                                                 *
 *                                                                            *
 * This software is owned by Microchip Technology Inc. ("Microchip") and is   *
 * supplied to you for use exclusively as described in the associated         *
 * software agreement.  This software is protected by software and other      *
 * intellectual property laws.  Any use in violation of the software license  *
 * may subject the user to criminal sanctions as well as civil liability.     *
 * Copyright 2006 Microchip Technology Inc.  All rights reserved.             *
 *                                                                            *
 * This software is provided "AS IS."  MICROCHIP DISCLAIMS ALL WARRANTIES,    *
 * EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, NOT LIMITED TO MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE, AND INFRINGEMENT. Microchip shall in no  *
 * event be liable for special, incidental, or consequential damages.         *
 *                                                                            *
 *- Version Log --------------------------------------------------------------*
 *   Date       Author        Comments                                        *
 *----------------------------------------------------------------------------*
 * 08/01/06 Howard Schlunder  Original                                        *
 * 04/25/07 Jorge Amodio      Changed file name to tcp_client_ex1.c           *
 ******************************************************************************/
#include "include/config.h"
#if defined(STACK_USE_TCP_EXAMPLE1)

#include "include/generic_typedefs.h"
#include "net/include/dns.h"
#include "net/include/helpers.h"
#include "net/include/tcp.h"
#include "net/include/arp.h"
#include "uart/include/uart.h"

//TODO: Define proper server address here
BYTE ServerName[] = "www.some_host.com";
WORD ServerPort = 80;

extern char szResponse[];

BYTE httpPost1[] = 	"POST /hidden.php HTTP/1.0\r\n"\
    "Host: some_host.com\r\n"
    "Content-type: application/x-www-form-urlencoded\r\n"
    "Content-length: ";
BYTE httpPost2[] = "\r\n\r\n"
	"s=";
BYTE httpPost3[] = "\r\n"
	"Connection: Close\r\n"
    "\r\n";

/******************************************************************************
 * Function:        void GenericTCPClient(void)
 * PreCondition:    Stack is initialized()
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        None
 * Note:            None
 ******************************************************************************/
void GenericTCPClient(void)
{
    BYTE i, *StringPtr;
    unsigned char tmp;
    char bufor[20];
    static TICK Timer;
    static TCP_SOCKET MySocket = INVALID_SOCKET;
    static NODE_INFO Server;
    static enum _GenericTCPExampleState
    {
        SM_HOME = 0,
        SM_NAME_RESOLVE,
        SM_ARP_START_RESOLVE,
        SM_ARP_RESOLVE,
        SM_SOCKET_OBTAIN,
        SM_SOCKET_OBTAINED,
        SM_PROCESS_RESPONSE,
        SM_DISCONNECT,
        SM_DONE
    } GenericTCPExampleState = SM_HOME;

    switch(GenericTCPExampleState)
    {
        case SM_HOME:
            // Obtain the IP address associated with the common ServerName
            DNSResolve(ServerName);
            GenericTCPExampleState++;
            break;

        case SM_NAME_RESOLVE:
            // Wait for the DNS server to return the requested IP address
            if(!DNSIsResolved(&Server.IPAddr))
                break;
            GenericTCPExampleState++;

        case SM_ARP_START_RESOLVE:
            // Obtain the MAC address associated with the server's IP address
            // (either direct MAC address on same subnet, or the MAC address
            // of the Gateway machine)
            ARPResolve(&Server.IPAddr);
            Timer = TickGet();
            GenericTCPExampleState++;
            break;

        case SM_ARP_RESOLVE:
            // Wait for the MAC address to finish being obtained
            if(!ARPIsResolved(&Server.IPAddr, &Server.MACAddr))
            {
                // Time out if too much time is spent in this state
                if(TickGet()-Timer > 1*TICK_SECOND)
                {
                    // Retransmit ARP request
                    GenericTCPExampleState--;
                }
                break;
            }
            GenericTCPExampleState++;

        case SM_SOCKET_OBTAIN:
            // Connect a socket to the remote TCP server
            MySocket = TCPConnect(&Server, ServerPort);
            
            // Abort operation if no TCP sockets are available
            // If this ever happens, incrementing MAX_TCP_SOCKETS in
            // config.h may help (at the expense of more global memory
            // resources).
            if(MySocket == INVALID_SOCKET)
                break;

            GenericTCPExampleState++;
            Timer = TickGet();
            break;

        case SM_SOCKET_OBTAINED:
            // Wait for the remote server to accept our connection request
            if(!TCPIsConnected(MySocket))
            {
                // Time out if too much time is spent in this state
                if(TickGet()-Timer > 5*TICK_SECOND)
                {
                    // Close the socket so it can be used by other modules
                    TCPDisconnect(MySocket);
                    MySocket = INVALID_SOCKET;
                    GenericTCPExampleState--;
                }
                break;
            }

            Timer = TickGet();

            // Make certain the socket can be written to
            if(!TCPIsPutReady(MySocket))
                break;
            
            // Place the application protocol data into the transmit buffer.
            // For this example, we are connected to an HTTP server, so we'll
            // send an HTTP POST request.
            for(i=0; i<strlen(httpPost1); i++)
            	TCPPut(MySocket, httpPost1[i]);
            itoa((int)(strlen(szResponse)+4), bufor);	//s=, %VA3, \r\n => strlen+4
			for(i=0; i<strlen(bufor); i++)
				TCPPut(MySocket, bufor[i]);
            for(i=0; i<strlen(httpPost2); i++)
            	TCPPut(MySocket, httpPost2[i]);
            for(i=0; i<strlen(szResponse); i++)
            	TCPPut(MySocket, szResponse[i]);
            for(i=0; i<strlen(httpPost3); i++)
            	TCPPut(MySocket, httpPost3[i]);

            // Send the packet
            TCPFlush(MySocket);
            GenericTCPExampleState++;

        case SM_PROCESS_RESPONSE:
            // Check to see if the remote node has disconnected from us or sent
            // us any application data
            // If application data is available, write it to the UART
            if(!TCPIsConnected(MySocket))
            {
                GenericTCPExampleState++;
            }

            if(!TCPIsGetReady(MySocket))
                break;

            // Obtain the server reply
            while(TCPGet(MySocket, &i))
            {
                while(BusyUART());
                WriteUART(i);
            }
            break;

        case SM_DISCONNECT:
            // Close the socket so it can be used by other modules
            // For this application, we wish to stay connected, but this state will
            // still get entered if the remote server decides to disconnect
            TCPDisconnect(MySocket);
            MySocket = INVALID_SOCKET;
            GenericTCPExampleState = SM_DONE;
            Timer = TickGet();
            break;

        case SM_DONE:
            if(TickGet()-Timer > (unsigned long)300*TICK_SECOND)
                GenericTCPExampleState = SM_HOME;
            break;
    }
}

#endif  // STACK_USE_TCP_EXAMPLE1
